package org.gcube.vremanagement.whnmanager.jaxws.ws;

import java.net.HttpURLConnection;
import java.util.Map.Entry;

import javax.jws.WebService;
import javax.ws.rs.core.Response.Status;

import org.gcube.common.gxhttp.request.GXHTTPStringRequest;
import org.gcube.common.gxhttp.util.ContentUtils;
import org.gcube.common.security.providers.SecretManagerProvider;
import org.gcube.common.security.secrets.Secret;
import org.gcube.resourcemanagement.whnmanager.api.WhnManager;
import org.gcube.resourcemanagement.whnmanager.api.exception.GCUBEUnrecoverableException;
import org.gcube.smartgears.ContextProvider;
import org.gcube.smartgears.context.application.ApplicationContext;
import org.gcube.smartgears.managers.ContextEvents;
import org.gcube.smartgears.security.SimpleCredentials;
import org.gcube.smartgears.security.defaults.DefaultAuthorizationProvider;
import org.gcube.vremanagement.whnmanager.utils.ValidationUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
//import org.gcube.common.calls.jaxws.Constants;

@WebService(portName = "WhnManagerIOPort", 
serviceName = WhnManager.SERVICE_NAME, 
targetNamespace = WhnManager.TNS, 
endpointInterface = "org.gcube.resourcemanagement.whnmanager.api.WhnManager" )
//@Singleton
public class WhnManagerImpl implements WhnManager{


	private static Logger logger=LoggerFactory.getLogger(WhnManagerImpl.class);

	/**
	 * Add a scope to the ghn profile and publish it on IS
	 */
	@Override
	public boolean addToContext(String context) throws GCUBEUnrecoverableException{
		Secret secret = SecretManagerProvider.instance.get();
		logger.trace("WHNManager: addToContext method invokation with parameters context :{} and caller: {} curentContext: {}",context, secret.getOwner().getId(), secret.getContext() );
		ValidationUtils.valid("context", context);
		ApplicationContext appContext = ContextProvider.get();
		if(context!=null){
			
			GXHTTPStringRequest request = GXHTTPStringRequest.newRequest("https://conductor.dev.d4science.org/api");
			try {

				request = request.path("workflow");
				for(Entry<String, String> entry : secret.getHTTPAuthorizationHeaders().entrySet())		
					request = request.header(entry.getKey(), entry.getValue());
					
				
				SimpleCredentials credentials = ((DefaultAuthorizationProvider) appContext.container().authorizationProvider()).getCredentials();
				
				HttpURLConnection response = request.post(String.format(" \"name\": \"ghn_client_add_to_contexts\",\n" + 
								" \"input\" : {\n" + 
								"     \"client_id\" : \"%s\",\n" + 
								"     \"context_list\" : [\"%s\"] }", credentials.getClientID(), context)) ;

				if(response.getResponseCode() == Status.CREATED.getStatusCode()) {
					String body =  ContentUtils.toString(ContentUtils.toByteArray(response.getInputStream()));
					logger.info("Returned response for remove scope {} ",body);
				}

				appContext.container().events().fire(context, ContextEvents.ADD_CONTEXT_TO_CONTAINER);
			} catch (Exception e) {
				logger.error("error adding context {}", context, e);
				return false;
			}
	
		}else{
			logger.error("context is null");
			return false;
		}
		return true;
	}

	/**
	 * Remove a scope from ghn profile and publish the new profile on IS
	 */
	@Override
	public boolean removeFromContext(String context)  throws GCUBEUnrecoverableException {
		Secret secret = SecretManagerProvider.instance.get();
		logger.trace("WHNManager: removeFromContext method invokation with parameters context :{} and caller: {} curentContext: {}",context, secret.getOwner().getId(), secret.getContext());
		ValidationUtils.valid("context", context);
		ApplicationContext appContext = ContextProvider.get();
		if(context!=null){
			logger.trace("allowed container in context are {} ",appContext.container().authorizationProvider().getContexts());
			
			GXHTTPStringRequest request = GXHTTPStringRequest.newRequest("https://conductor.dev.d4science.org/api");
			try {

				request = request.path("workflow");
				for(Entry<String, String> entry : secret.getHTTPAuthorizationHeaders().entrySet())		
					request = request.header(entry.getKey(), entry.getValue());
					
				
				SimpleCredentials credentials = ((DefaultAuthorizationProvider) appContext.container().authorizationProvider()).getCredentials();
				
				HttpURLConnection response = request.post(String.format(" \"name\": \"ghn_client_remove_from_contexts\",\n" + 
								" \"input\" : {\n" + 
								"     \"client_id\" : \"%s\",\n" + 
								"     \"context_list\" : [\"%s\"] }", credentials.getClientID(), context)) ;

				if(response.getResponseCode() == Status.CREATED.getStatusCode()) {
					String body =  ContentUtils.toString(ContentUtils.toByteArray(response.getInputStream()));
					logger.info("Returned response for remove scope {} ",body);
				}

				appContext.container().events().fire(context, ContextEvents.REMOVE_CONTEXT_FROM_CONTAINER);
			} catch (Exception e) {
				logger.error("error removing context {}", context, e);
				return false;
			}


		}else{
			logger.error("context is null");
			return false;
		}
		return true;
	}


}
